#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## file_object.pm for aspiq plugins
## 
## ######### PROJECT DESCRIPTION : ###
## 
## This file is an aspiq object. It allows user to
## read/write and make actions on files with root rights.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Jan 31 17:00:43 2008 Boutonnet Alexandre
## Last update Wed Oct 29 18:10:39 2008 Boutonnet Alexandre
## 
## ###################################
##

package file;

use strict;
use warnings;

use Data::Dumper;
use File::Path;

my $MAX_READ_LENGTH = 6000000; # 6 Mo

# Error tab
my $error_hash = {
    0 => 'Success',
    100 => 'File does not exist',
    101 => 'File is not readable',
    102 => 'Unable to delete this file',
    103 => 'File is a directory',
    104 => 'This file is not readable with this command',
    105 => 'File is a directory, use "force" for delete it anyway',
    106 => 'Deleting this type of file is not allowed',
    107 => 'This file is not a regular file',
    108 => 'Content is not defined',
    109 => 'Unable to open file : ',
    110 => 'An error occured during the write process',
    111 => 'Filename parameter not found',
    200 => 'Read only mode : this command can not be executed',
    201 => 'This file is bigger than the limit : '.$MAX_READ_LENGTH.' bytes',
    202 => 'File is empty',
    300 => 'This method does nothing in this plugin',
};

# This function is the constructor of this object
# The parameter of this method is an instance of the
# log object.
sub new
{
    my $log_obj = shift; # Getting the log object.

    my $OBJ = {}; # This had to contain object's personnal datas.

    $OBJ->{log} = $log_obj;

    bless($OBJ);

    return($OBJ);
}

# This function permit to user to initiate this object
# The parameter of this method is the read-only boolean
sub init
{
    my ($OBJ, $read_only) = @_;

    $OBJ->{read_only} = $read_only;

    return(0);
}

# This method permit to user to get info
# The parameter is an hash containing parameters.
sub get
{
    my ($OBJ, $hash) = @_;

    # Testing if hash contains filename key
    return(_make_error_hash(111, $error_hash->{111})) if (!exists($hash->{'filename'}));

    my $file = $hash->{'filename'}[0];

    # Testing if file exist
    return(_make_error_hash(100, $error_hash->{100})) if (! -e $file);

    # Testing if the file is readable with this method
    return(_make_error_hash(103, $error_hash->{103})) if (-d $file);
    return(_make_error_hash(104, $error_hash->{104})) if (-p $file || -S $file ||
							  -b $file || -c $file || -t $file);

    # Testing if file is readable
    return(_make_error_hash(101, $error_hash->{101})) if (! -r $file);

    # Testing if file is empty
    return(_make_error_hash(202, $error_hash->{202})) if (-z $file);

    # Testing file size
    return(_make_error_hash(201, $error_hash->{201})) if (-s $file > $MAX_READ_LENGTH);

    open(FD, $file) or _make_error_hash(109, $error_hash->{109});

    my $buffer = '';
    my $size = 0;
    while(<FD>)
    {
	my $line = $_;
	$buffer .= $line;
	$size += length($line);
    }

    my $hash_return = {
	'content' => [
		      $buffer
		      ],
		      'read_bytes' => [
				       $size
				       ],
				   };
    return($hash_return);
}

# This method permit to user to make an action
# The parameter is an hash containing parameters.
sub set
{
    my ($OBJ, $hash) = @_;

    # Testing if hash contains filename key
    return(_make_error_hash(111, $error_hash->{111})) if (!exists($hash->{'filename'}));
    
    my $file = $hash->{'filename'}[0];
    
    if (-e $file)
    {
	# Testing if file is writable
	return(_make_error_hash(101, $error_hash->{101})) if (! -w $file);
	
	# Testing if file is a regular file
	return(_make_error_hash(107, $error_hash->{107})) if (! -f $file);
    }

    return(_make_error_hash(108, $error_hash->{108})) if (!exists($hash->{'content'}));
    
    open(FD, "+>", $file) or return (_make_error_hash(109, $error_hash->{109}."$!"));

    my $retour = print FD $hash->{'content'}[0];
    close(FD);

    return(_make_error_hash(110, $error_hash->{110})) if (!$retour);

    return(_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to get all info
# There is no parameter to this method.
sub info
{
    return(_make_error_hash(300, $error_hash->{300}));
}

# This method permit to the user to delete an entity.
# The parameter is an hash containing parameters.
sub delete
{
    my ($OBJ, $hash) = @_;

    # Testing if hash contains filename key
    return(_make_error_hash(111, $error_hash->{111})) if (!exists($hash->{'filename'}));

    my $file = $hash->{'filename'}[0];

    # Testing if file exist
    return(_make_error_hash(100, $error_hash->{100})) if (! -e $file);

    # Testing if file is readable
    return(_make_error_hash(101, $error_hash->{101})) if (! -r $file);

    # Testing if the file is a directory
    return(_make_error_hash(105, $error_hash->{105})) if (-d $file && (!exists($hash->{'force'})
								       || $hash->{'force'}[0] == 0));

    # Can I delete this file ?
    return(_make_error_hash(106, $error_hash->{106})) if (-p $file || -S $file ||
							  -b $file || -c $file || -t $file);

    if (-d $file)
    {
	# First of all, the used version of rmtree is 1.08 that is very
	# deprecated (thanks debian/perl 5.8.8).
	# Then using rmtree create a beautiful race condition (in addition
	# this script runs with root right).
	return(_make_error_hash(102, $error_hash->{102})) if (!rmtree($file, 0, 0));
    }
    else
    {
	# Deleting file...
	return(_make_error_hash(102, $error_hash->{102})) if (!unlink($file));
    }

    return(_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to get all available method
# There is no parameter.
sub list_methods
{
    my $OBJ = shift;

    my $hash_return = {
	'methods' => [
		      'list_error_codes',
		      'get',
		      'stat',
		      ],
	};
    
    push(@{$hash_return->{'methods'}}, "set") if(!$OBJ->{read_only});
    push(@{$hash_return->{'methods'}}, "delete") if(!$OBJ->{read_only});

    return($hash_return);
}


# This method permit to the user to get all error codes.
# There is no parameter.
sub list_error_codes
{
    my $hash_return = {
	'error_codes' => []
    };

    foreach my $key (sort keys(%{$error_hash}))
    {
	my $tmp_hash = {
	    'error' => [$key],
	    'msg' => [$error_hash->{$key}],
	};
	push(@{$hash_return->{'error_codes'}}, $tmp_hash);
    }

    return($hash_return);
}

### Unstandard methods

# This method permit to the user to get all information on the file
sub stat
{
    my ($OBJ, $hash) = @_;

    # Testing if hash contains filename key
    return(_make_error_hash(111, $error_hash->{111})) if (!exists($hash->{'filename'}));

    my $file = $hash->{'filename'}[0];
    
    # Testing if file exist
    return(_make_error_hash(100, $error_hash->{100})) if (! -e $file);
    
    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
     $atime,$mtime,$ctime,$blksize,$blocks) = stat($file);

    my $hash_return = {
	'device_number' => [ $dev ],
	'inode_number' => [ $ino ],
	'mode' => [ $mode ],
	'nlink' => [ $nlink ],
	'uid' => [ $uid ],
	'gid' => [ $gid ],
	'rdev' => [ $rdev ],
	'size' => [ $size ],
	'atime' => [ $atime ],
	'mtime' => [ $mtime ],
	'ctime' => [ $ctime ],
	'blksize' => [ $blksize ],
	'blocks' => [ $blocks ],
    };
    
    return($hash_return);
}

### Private methods
sub _make_error_hash
{
    my ($error_code, $error_msg) = @_;

    my $hash_error = {
	'error_code' => [$error_code],
	'error_msg' => [$error_msg]
	};

    return($hash_error);
}

1;
